# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-09-16 09:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="AuthKey",
            fields=[
                (
                    "id",
                    models.CharField(max_length=40, primary_key=True, serialize=False),
                ),
                ("uid", models.PositiveIntegerField(null=True)),
                ("expired", models.DateTimeField()),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("onetime", models.BooleanField(default=True)),
                ("data", models.TextField()),
            ],
        ),
    ]
